---
title: "Getting started with clusters"
author: "Jean Feydy"
institute: ""
topic: ""
theme: "metropolis"
colortheme:
fonttheme:
fontsize: 11pt
linkcolor: black
urlcolor: blue
linkstyle: bold
aspectratio: 169
titlegraphic: 
logo:
date:
section-titles: false
toc: false
---


## Today's talk


**Why use a cluster?**

- **Fast** run times.
- **Large** scale data.
- **Reproducible** experiments.


**Feel free to ask me your questions:**

- I use clusters **weekly**.
- INRIA Paris: delegate for HeKA at the users' committee.
- INRIA clusters: delegate for Paris at the users' committee.
- Happy to **bring you all** fast run times.


## Today's talk

**1. Connect to a distant machine with SSH**

- SSH authentication
- File transfer with ssh:// or sftp://
- Full remote development with VSCode

**2. Clusters that I use regularly** -- **no data security**: open or virtual cohorts only!

- Amazon Web Service EC2
- Cleps at INRIA Paris
- Jean Zay, the French cluster for AI research

**3. Set up your environment**

- pip? conda? Docker?
- Singularity


# SSH connection


## How to use SSH

**Standard** tool for remote access:

- Linux and MacOS: $\texttt{ssh}$ is already installed.
- Windows: $\texttt{OpenSSH}$ is officially supported by Microsoft, but opt-in.

Typical commands to open a remote terminal:

- $\texttt{ssh user@serverip}~~~$ with password.
- $\texttt{ssh myfavoritecluster}~~~$ without password.

I use $\texttt{tmux}$ to create **"immortal" shells** on the cluster that stay alive even if \
I close my session or experience connection issues.


```{=latex}
\end{frame}

\begin{frame}[fragile]{SSH config: ~~\texttt{\char`~/.ssh/config} ~~or~~ \texttt{\%userprofile\%\textbackslash{}.ssh\textbackslash{}config}}

\vspace*{.5cm}

\begin{minted}[bgcolor=blue!4,mathescape,tabsize=4,firstnumber=1,stripnl=false,fontsize=\small,linenos]{bash}
# Dummy local connection:
Host localhost
HostName 127.0.0.1
User jean

# Cleps - Inria Paris:
Host cleps
User jfeydy
HostName cleps.inria.fr
port 22
ControlMaster auto
ForwardX11 no
ForwardAgent yes
\end{minted}


\end{frame}

\begin{frame}[fragile]{SSH config: ~~\texttt{\char`~/.ssh/config} ~~or~~ \texttt{\%userprofile\%\textbackslash{}.ssh\textbackslash{}config}}

\vspace*{.5cm}

\begin{minted}[bgcolor=blue!4,mathescape,tabsize=4,firstnumber=14,stripnl=false,fontsize=\small,linenos]{bash}
# Inria bastion for Jean Zay:
Host inria-ssh
User jfeydy
HostName ssh.paris.inria.fr

# Jean Zay:
Host jean-zay
User my_personal_id
HostName jean-zay.idris.fr
ProxyJump inria-ssh



\end{minted}
\end{frame}



\begin{frame}[fragile]{Avoid passwords with a secure key: ~~\texttt{\char`~/.ssh/id\_rsa} ~~and~~ \texttt{id\_rsa.pub}}

\vspace*{.5cm}

\begin{minted}[bgcolor=blue!4,mathescape,tabsize=4,firstnumber=1,stripnl=false,fontsize=\small,linenos]{bash}
# Create your id_rsa, if you don't have one already:
ssh-keygen -t rsa

# Ensure that your favorite servers remember you:
ssh-copy-id id@server
\end{minted}
```


## $\texttt{ssh://cleps/home/jfeydy~~}$ or $\texttt{~~sftp://...~~}$ in the address bar 

`\begin{center}`{=latex}
![](images/SFTP.png){height=90%}
`\end{center}`{=latex}


## Full remote integration with Visual Studio Code

`\begin{center}`{=latex}
![](images/VSCode.png){height=100%}
`\end{center}`{=latex}


# Getting access to a cluster

## Paperwork-free option: rent an instance on AWS EC2

`\begin{center}`{=latex}
![](images/AWS_1.png){height=100%}
`\end{center}`{=latex}

## Get full access to a custom configuration

`\begin{center}`{=latex}
![](images/AWS_2.png){height=100%}
`\end{center}`{=latex}


## All types of hardware are available

`\begin{center}`{=latex}
![](images/AWS_3.png){height=100%}
`\end{center}`{=latex}

## Typical prices: about 3€ per hour for a high-end GPU

`\begin{center}`{=latex}
![](images/AWS_4.png){height=100%}
`\end{center}`{=latex}


## What about INRIA?

**Commercial** cloud services:

- Great for **reproducible benchmarks**.
- A **"marché public"** for AWS, GCP, OVH is currently being approved.
- Until then: pay in advance from your **own pocket**, \
  get a re-fund from your **research grants** as "menus frais" for up to **500€ / month**.


**INRIA** clusters:

- Local cluster at INRIA Paris: Cleps.
- Currently being merged within the Grid5k framework.


**National** cluster for **AI research**:

- Jean Zay: lots of **high-end GPUs** and **dedicated support**, free of charge.
- Main drawback: super strict on **security**.


## Cleps: a local cluster at INRIA Paris


**Full documentation** is available: \
https://paris-cluster-2019.gitlabpages.inria.fr/cleps/cleps-userguide/index.html

Three conditions:

- Have a $~~\texttt{jfeydy@inria.fr}~~$ account.
 
- Use your **professional** computer.

- Connect to the **INRIA network** with a VPN:\
  $\texttt{sudo openconnect -u jfeydy vpn.inria.fr}$


## On INRIA laptops: just use the graphical VPN menu

`\begin{center}`{=latex}
![](images/VPN.png){height=95%}
`\end{center}`{=latex}


```{=latex}
\end{frame}

\begin{frame}[fragile]{SSH config: ~~\texttt{\char`~/.ssh/config} ~~or~~ \texttt{\%userprofile\%\textbackslash{}.ssh\textbackslash{}config}}

\vspace*{.5cm}

\begin{minted}[bgcolor=blue!4,mathescape,tabsize=4,firstnumber=1,stripnl=false,fontsize=\small,linenos]{bash}
# Dummy local connection:
Host localhost
HostName 127.0.0.1
User jean

# Cleps - Inria Paris:
Host cleps
User jfeydy
HostName cleps.inria.fr
port 22
ControlMaster auto
ForwardX11 no
ForwardAgent yes
\end{minted}
```


## $\texttt{ssh cleps}$

`\begin{center}`{=latex}
![](images/CLEPS_1.png){height=100%}
`\end{center}`{=latex}


```{=latex}
\end{frame}

\begin{frame}[fragile]{The Slurm manager: launch scripts with $~~~\texttt{sbatch myproject.batch}$}


\begin{minted}[bgcolor=blue!4,mathescape,tabsize=4,firstnumber=1,stripnl=false,fontsize=\footnotesize,linenos]{bash}
#!/bin/bash
#SBATCH --job-name=myproject     # create a short name for your job
#SBATCH --mail-type=ALL          # mail events: NONE, BEGIN, END, FAIL...
#SBATCH --mail-user=jean.feydy@inria.fr
#SBATCH --nodes=1                # node count
#SBATCH --ntasks=1               # total number of tasks across all nodes
#SBATCH --cpus-per-task=16       # cpu-cores per task
#SBATCH --partition=gpu          # Name of the partition
#SBATCH --gres=gpu:rtx6000:1     # Use a GPU node
#SBATCH --mem=40G                # Total memory allocated
#SBATCH --hint=multithread       # we get physical cores not logical
#SBATCH --time=03:00:00          # total run time limit (HH:MM:SS)
#SBATCH --output=logs/myproject.out   # output file name

echo "### Running $SLURM_JOB_NAME ###"
cd ${SLURM_SUBMIT_DIR}
your bash script...
\end{minted}
```

## Jean Zay: the French AI cluster

Ideal for AI research on non-sensitive data:

- **90k** high-end CPU cores with **450 TB of RAM**.
- **2,700 V100** GPUs + **416 A100** GPUs.
- **Admins** who understand the needs of AI/ML researchers: see e.g.\
  http://www.idris.fr/jean-zay/pre-post/jean-zay-jupyter-notebook.html


Full documentation is available:

- Paperwork and tutorials: https://jean-zay-doc.readthedocs.io/
- Technical documentation: http://www.idris.fr/jean-zay/


## Getting access to Jean Zay

My main steps -- Summer 2022:

1. **Follow** https://jean-zay-doc.readthedocs.io/en/latest/access-procedure/
2. **Asked** for 8,000 V100 hours + 2,000 A100 hours per year.
3. Ask Eric Fleury to sign the **"Visa request"** \
   via his assistant $\texttt{celine.scherschen@inria.fr}$.
4. Ask a **"security stamp"** to $\texttt{Laurent.Le\_Pendeven@inria.fr}$.
5. Tell Jean Zay that I will connect from the **INRIA IP address**: $\texttt{ssh.paris.inria.fr}~$ = $~\texttt{128.93.96.2}$.
6. Create a "ticket" on the INRIA system to add my SSH $\texttt{id\_rsa.pub}$ \
  to the **list of known users** on $\texttt{ssh.paris.inria.fr}$. \
  Dedicated category: "Demande d'accès distants serveurs : Accès Bastion SSH".

```{=latex}
\end{frame}

\begin{frame}[fragile]{SSH config: ~~\texttt{\char`~/.ssh/config} ~~or~~ \texttt{\%userprofile\%\textbackslash{}.ssh\textbackslash{}config}}

\vspace*{.5cm}

\begin{minted}[bgcolor=blue!4,mathescape,tabsize=4,firstnumber=14,stripnl=false,fontsize=\small,linenos]{bash}
# Inria bastion for Jean Zay:
Host inria-ssh
User jfeydy
HostName ssh.paris.inria.fr

# Jean Zay:
Host jean-zay
User my_personal_id
HostName jean-zay.idris.fr
ProxyJump inria-ssh



\end{minted}
```


# Set up a reproducible environment

## Common dependency managers

**Virtual environment** with Python -- to handle conflicts:

- **pip**: for pure Python packages.
- **conda**: to handle C/C++/binary extensions.

**Containers** -- fast and lightweight virtual machines to ensure portability:

- **Docker**: industry standard.
- **Singularity**: standard on shared academic clusters, \
  more secure but more rigid, 100% compatible with Docker.


## Singularity

**Install** Singularity from https://github.com/sylabs/singularity/releases \
or use a command such as "$\texttt{module load singularity}$" (on a cluster).

**Full documentation** is available online. You may:

- Define precisely **your own configuration** with a bash install script.
- Keep your environment clean with a custom "Home" folder per project. \
  This is especially important to ensure **full reproducibility**.
- Connect to your containers via SSH to benefit from VSCode integration, etc.


```{=latex}
\end{frame}

\begin{frame}[fragile]{Install a configuration that you like from DockerHub}

\begin{minted}[bgcolor=blue!4,mathescape,tabsize=4,firstnumber=14,stripnl=false,fontsize=\small,linenos]{bash}
# For large images, the in-memory tmpfs may be too small:
# we create a folder on the hard drive instead.
mkdir tmp
# Perform a virtual install of the official KeOps environment
# (Ubuntu + CUDA + PyTorch + KeOps + survival-GPU + ...)
# in the immutable file image.sif (~6 GB):
SINGULARITY_TMPDIR=`pwd`/tmp \
singularity build image.sif docker://getkeops/keops-full:latest

# Note that this requires admin rights and may take some time:
# you may perform this step on your laptop, and copy-paste
# image.sif on the cluster afterwards.

# Then, run a shell in the virtual machine with NVidia GPU support:
singularity shell --nv image.sif
\end{minted}

\end{frame} 

\begin{frame}[standout]
Please feel free to ask me questions anytime!
```


